/**
 * \addtogroup keymgr
 */
/*@{*/


/***************************************************
 * File name: keymgr_crypto.c
 *
 *
 ***************************************************/
/** \file keymgr_crypto.c
 *
 * \brief This is the implementation of Cryptography algorightms for Homeplug AV.
 *
 * Implementation of the FIPS 180-2 SHA-256 hash algorithm
 * Implementation of the PKCS #5 PBKDF1 key derivation function
 * Implementation of the HMAC SHA-256 algorithm
 *
 **************************************************/

/*
 * FIPS 180-2 SHA-224/256/384/512 implementation
 * Last update: 02/02/2007
 * Issue date:  04/30/2005
 *
 * Copyright (C) 2005, 2007 Olivier Gay <olivier.gay@a3.epfl.ch>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */



/***************************************************
*                 Include section
***************************************************/

#include <string.h>

#include "gigtypes.h"         /* General types */

#include "keymgr_crypto.h"    /* Constants for cryptographic functions */


/***************************************************
 *                 Local Constants Section
 ***************************************************/


/***************************************************
 *                 Local Defines Section
 ***************************************************/

/** \brief Shift right function */
#define SHFR(x, n)    (x >> n)
/** \brief Rotate right function */
#define ROTR(x, n)   ((x >> n) | (x << ((sizeof(x) << 3) - n)))
/** \brief Rotate left function */
#define ROTL(x, n)   ((x << n) | (x >> ((sizeof(x) << 3) - n)))
/** \brief SHA-256 Ch function */
#define CH(x, y, z)  ((x & y) ^ (~x & z))
/** \brief SHA-256 Maj function */
#define MAJ(x, y, z) ((x & y) ^ (x & z) ^ (y & z))
/** \brief SHA-256 Sigma0 function */
#define SHA256_F1(x) (ROTR(x,  2) ^ ROTR(x, 13) ^ ROTR(x, 22))
/** \brief SHA-256 Sigma1 function */
#define SHA256_F2(x) (ROTR(x,  6) ^ ROTR(x, 11) ^ ROTR(x, 25))
/** \brief SHA-256 Gamma0 function */
#define SHA256_F3(x) (ROTR(x,  7) ^ ROTR(x, 18) ^ SHFR(x,  3))
/** \brief SHA-256 Gamma1 function */
#define SHA256_F4(x) (ROTR(x, 17) ^ ROTR(x, 19) ^ SHFR(x, 10))

/** \brief 32-bit integer manipulation macros */
#define UNPACK32(x, str)                      \
{                                             \
  *((str) + 3) = (t_uint8) ((x)      );       \
  *((str) + 2) = (t_uint8) ((x) >>  8);       \
  *((str) + 1) = (t_uint8) ((x) >> 16);       \
  *((str) + 0) = (t_uint8) ((x) >> 24);       \
}

/** \brief 32-bit integer manipulation macros */
#define PACK32(str, x)                    \
{                                         \
  *(x) = ((t_uint32) *((str) + 3))        \
    | ((t_uint32) *((str) + 2) <<  8)     \
    | ((t_uint32) *((str) + 1) << 16)     \
    | ((t_uint32) *((str) + 0) << 24);    \
}

/** \brief Macro for SHA-256 expansion */
#define SHA256_SCR(i)                       \
{                                           \
  w[i] =  SHA256_F4(w[i -  2]) + w[i - 7]   \
    + SHA256_F3(w[i - 15]) + w[i - 16];     \
}

/** \brief Macro used for loops unrolling */
#define SHA256_EXP(a, b, c, d, e, f, g, h, j)             \
{                                                         \
  t1 = wv[h] + SHA256_F2(wv[e]) + CH(wv[e], wv[f], wv[g]) \
    + sha256_k[j] + w[j];                                 \
  t2 = SHA256_F1(wv[a]) + MAJ(wv[a], wv[b], wv[c]);       \
  wv[d] += t1;                                            \
  wv[h] = t1 + t2;                                        \
}


/***************************************************
 *                 Local Typedefs Section
 ***************************************************/


/***************************************************
 *                 Local Variables Section
 ***************************************************/

/** \brief contain the initial hash value H(0) */
const t_uint32 sha256_h0[8] =
  {0x6a09e667, 0xbb67ae85, 0x3c6ef372, 0xa54ff53a,
   0x510e527f, 0x9b05688c, 0x1f83d9ab, 0x5be0cd19};

/** \brief array of costants used by the compression function */
const t_uint32 sha256_k[64] =
  {0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5,
   0x3956c25b, 0x59f111f1, 0x923f82a4, 0xab1c5ed5,
   0xd807aa98, 0x12835b01, 0x243185be, 0x550c7dc3,
   0x72be5d74, 0x80deb1fe, 0x9bdc06a7, 0xc19bf174,
   0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
   0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da,
   0x983e5152, 0xa831c66d, 0xb00327c8, 0xbf597fc7,
   0xc6e00bf3, 0xd5a79147, 0x06ca6351, 0x14292967,
   0x27b70a85, 0x2e1b2138, 0x4d2c6dfc, 0x53380d13,
   0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
   0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3,
   0xd192e819, 0xd6990624, 0xf40e3585, 0x106aa070,
   0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5,
   0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,
   0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
   0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2};


/**************************************************
 *  Function Prototype Section
 * Add prototypes for all local functions defined
 * in this file
 ***************************************************/

void sha256_transf(sha256_ctx *ctx, const unsigned char *message,
    t_uint32 block_nb);
void sha256_init(sha256_ctx *ctx);
void sha256_update(sha256_ctx *ctx, const unsigned char *message,
    t_uint32 len);
void sha256_final(sha256_ctx *ctx, unsigned char *digest);



/* SHA-256 functions */

/** \brief expand and compress the message */
void sha256_transf(sha256_ctx *ctx, const unsigned char *message,
    t_uint32 block_nb)
{
  t_uint32 w[64];
  t_uint32 wv[8];
  t_uint32 t1, t2;
  const unsigned char *sub_block;
  t_int32 i;

  #ifndef UNROLL_LOOPS
    t_int32 j;
  #endif
    for (i = 0; i < (t_int32) block_nb; i++)
    {
      /* offset of block number i */
      sub_block = message + (i << 6);

  #ifndef UNROLL_LOOPS
    /* make SHA-256 expansion */
    for (j = 0; j < 16; j++)
    {
      PACK32(&sub_block[j << 2], &w[j]);
    }

    for (j = 16; j < 64; j++)
    {
      SHA256_SCR(j);
    }

    for (j = 0; j < 8; j++)
    {
      wv[j] = ctx->h[j];
    }

    /* make SHA-256 compression */
    for (j = 0; j < 64; j++)
    {
      t1 = wv[7] + SHA256_F2(wv[4]) + CH(wv[4], wv[5], wv[6])
          + sha256_k[j] + w[j];
      t2 = SHA256_F1(wv[0]) + MAJ(wv[0], wv[1], wv[2]);
      wv[7] = wv[6];
      wv[6] = wv[5];
      wv[5] = wv[4];
      wv[4] = wv[3] + t1;
      wv[3] = wv[2];
      wv[2] = wv[1];
      wv[1] = wv[0];
      wv[0] = t1 + t2;
    }

    for (j = 0; j < 8; j++)
    {
      ctx->h[j] += wv[j];
    }
  #else
    PACK32(&sub_block[ 0], &w[ 0]);
    PACK32(&sub_block[ 4], &w[ 1]);
    PACK32(&sub_block[ 8], &w[ 2]);
    PACK32(&sub_block[12], &w[ 3]);
    PACK32(&sub_block[16], &w[ 4]);
    PACK32(&sub_block[20], &w[ 5]);
    PACK32(&sub_block[24], &w[ 6]);
    PACK32(&sub_block[28], &w[ 7]);
    PACK32(&sub_block[32], &w[ 8]);
    PACK32(&sub_block[36], &w[ 9]);
    PACK32(&sub_block[40], &w[10]);
    PACK32(&sub_block[44], &w[11]);
    PACK32(&sub_block[48], &w[12]);
    PACK32(&sub_block[52], &w[13]);
    PACK32(&sub_block[56], &w[14]);
    PACK32(&sub_block[60], &w[15]);

    SHA256_SCR(16); SHA256_SCR(17); SHA256_SCR(18); SHA256_SCR(19);
    SHA256_SCR(20); SHA256_SCR(21); SHA256_SCR(22); SHA256_SCR(23);
    SHA256_SCR(24); SHA256_SCR(25); SHA256_SCR(26); SHA256_SCR(27);
    SHA256_SCR(28); SHA256_SCR(29); SHA256_SCR(30); SHA256_SCR(31);
    SHA256_SCR(32); SHA256_SCR(33); SHA256_SCR(34); SHA256_SCR(35);
    SHA256_SCR(36); SHA256_SCR(37); SHA256_SCR(38); SHA256_SCR(39);
    SHA256_SCR(40); SHA256_SCR(41); SHA256_SCR(42); SHA256_SCR(43);
    SHA256_SCR(44); SHA256_SCR(45); SHA256_SCR(46); SHA256_SCR(47);
    SHA256_SCR(48); SHA256_SCR(49); SHA256_SCR(50); SHA256_SCR(51);
    SHA256_SCR(52); SHA256_SCR(53); SHA256_SCR(54); SHA256_SCR(55);
    SHA256_SCR(56); SHA256_SCR(57); SHA256_SCR(58); SHA256_SCR(59);
    SHA256_SCR(60); SHA256_SCR(61); SHA256_SCR(62); SHA256_SCR(63);

    wv[0] = ctx->h[0]; wv[1] = ctx->h[1];
    wv[2] = ctx->h[2]; wv[3] = ctx->h[3];
    wv[4] = ctx->h[4]; wv[5] = ctx->h[5];
    wv[6] = ctx->h[6]; wv[7] = ctx->h[7];

    SHA256_EXP(0,1,2,3,4,5,6,7, 0); SHA256_EXP(7,0,1,2,3,4,5,6, 1);
    SHA256_EXP(6,7,0,1,2,3,4,5, 2); SHA256_EXP(5,6,7,0,1,2,3,4, 3);
    SHA256_EXP(4,5,6,7,0,1,2,3, 4); SHA256_EXP(3,4,5,6,7,0,1,2, 5);
    SHA256_EXP(2,3,4,5,6,7,0,1, 6); SHA256_EXP(1,2,3,4,5,6,7,0, 7);
    SHA256_EXP(0,1,2,3,4,5,6,7, 8); SHA256_EXP(7,0,1,2,3,4,5,6, 9);
    SHA256_EXP(6,7,0,1,2,3,4,5,10); SHA256_EXP(5,6,7,0,1,2,3,4,11);
    SHA256_EXP(4,5,6,7,0,1,2,3,12); SHA256_EXP(3,4,5,6,7,0,1,2,13);
    SHA256_EXP(2,3,4,5,6,7,0,1,14); SHA256_EXP(1,2,3,4,5,6,7,0,15);
    SHA256_EXP(0,1,2,3,4,5,6,7,16); SHA256_EXP(7,0,1,2,3,4,5,6,17);
    SHA256_EXP(6,7,0,1,2,3,4,5,18); SHA256_EXP(5,6,7,0,1,2,3,4,19);
    SHA256_EXP(4,5,6,7,0,1,2,3,20); SHA256_EXP(3,4,5,6,7,0,1,2,21);
    SHA256_EXP(2,3,4,5,6,7,0,1,22); SHA256_EXP(1,2,3,4,5,6,7,0,23);
    SHA256_EXP(0,1,2,3,4,5,6,7,24); SHA256_EXP(7,0,1,2,3,4,5,6,25);
    SHA256_EXP(6,7,0,1,2,3,4,5,26); SHA256_EXP(5,6,7,0,1,2,3,4,27);
    SHA256_EXP(4,5,6,7,0,1,2,3,28); SHA256_EXP(3,4,5,6,7,0,1,2,29);
    SHA256_EXP(2,3,4,5,6,7,0,1,30); SHA256_EXP(1,2,3,4,5,6,7,0,31);
    SHA256_EXP(0,1,2,3,4,5,6,7,32); SHA256_EXP(7,0,1,2,3,4,5,6,33);
    SHA256_EXP(6,7,0,1,2,3,4,5,34); SHA256_EXP(5,6,7,0,1,2,3,4,35);
    SHA256_EXP(4,5,6,7,0,1,2,3,36); SHA256_EXP(3,4,5,6,7,0,1,2,37);
    SHA256_EXP(2,3,4,5,6,7,0,1,38); SHA256_EXP(1,2,3,4,5,6,7,0,39);
    SHA256_EXP(0,1,2,3,4,5,6,7,40); SHA256_EXP(7,0,1,2,3,4,5,6,41);
    SHA256_EXP(6,7,0,1,2,3,4,5,42); SHA256_EXP(5,6,7,0,1,2,3,4,43);
    SHA256_EXP(4,5,6,7,0,1,2,3,44); SHA256_EXP(3,4,5,6,7,0,1,2,45);
    SHA256_EXP(2,3,4,5,6,7,0,1,46); SHA256_EXP(1,2,3,4,5,6,7,0,47);
    SHA256_EXP(0,1,2,3,4,5,6,7,48); SHA256_EXP(7,0,1,2,3,4,5,6,49);
    SHA256_EXP(6,7,0,1,2,3,4,5,50); SHA256_EXP(5,6,7,0,1,2,3,4,51);
    SHA256_EXP(4,5,6,7,0,1,2,3,52); SHA256_EXP(3,4,5,6,7,0,1,2,53);
    SHA256_EXP(2,3,4,5,6,7,0,1,54); SHA256_EXP(1,2,3,4,5,6,7,0,55);
    SHA256_EXP(0,1,2,3,4,5,6,7,56); SHA256_EXP(7,0,1,2,3,4,5,6,57);
    SHA256_EXP(6,7,0,1,2,3,4,5,58); SHA256_EXP(5,6,7,0,1,2,3,4,59);
    SHA256_EXP(4,5,6,7,0,1,2,3,60); SHA256_EXP(3,4,5,6,7,0,1,2,61);
    SHA256_EXP(2,3,4,5,6,7,0,1,62); SHA256_EXP(1,2,3,4,5,6,7,0,63);

    ctx->h[0] += wv[0]; ctx->h[1] += wv[1];
    ctx->h[2] += wv[2]; ctx->h[3] += wv[3];
    ctx->h[4] += wv[4]; ctx->h[5] += wv[5];
    ctx->h[6] += wv[6]; ctx->h[7] += wv[7];
  #endif /* !UNROLL_LOOPS */
  }
}

/** \brief initialize the SHA-256 state to the initial state */
void sha256_init(sha256_ctx *ctx)
{
  #ifndef UNROLL_LOOPS
    t_int32 i;
    for (i = 0; i < 8; i++)
    {
      ctx->h[i] = sha256_h0[i];
    }
  #else
    ctx->h[0] = sha256_h0[0];
    ctx->h[1] = sha256_h0[1];
    ctx->h[2] = sha256_h0[2];
    ctx->h[3] = sha256_h0[3];
    ctx->h[4] = sha256_h0[4];
    ctx->h[5] = sha256_h0[5];
    ctx->h[6] = sha256_h0[6];
    ctx->h[7] = sha256_h0[7];
  #endif /* !UNROLL_LOOPS */

  ctx->len = 0;
  ctx->tot_len = 0;
}

/** \brief process message: perform hashing on intermediate blocks */
void sha256_update(sha256_ctx *ctx, const unsigned char *message,
    t_uint32 len)
{
  /* number of 512-bits blocks - 1 */
  t_uint32 block_nb;
  t_uint32 new_len, rem_len, tmp_len;
  const unsigned char *shifted_message;

  tmp_len = SHA256_BLOCK_SIZE - ctx->len;
  rem_len = len < tmp_len ? len : tmp_len;

  /* if the message length is less than one block copy it all, otherwise copy
   * one block */
  memcpy(&ctx->block[ctx->len], message, rem_len);

  /* if the message length is less than one block go to the final function */
  if (ctx->len + len < SHA256_BLOCK_SIZE)
  {
    ctx->len += len;
    return;
  }

  new_len = len - rem_len;
  block_nb = new_len / SHA256_BLOCK_SIZE;

  shifted_message = message + rem_len;

  /* hash the first 512 bits block */
  sha256_transf(ctx, ctx->block, 1);
  /* hash the following blocks until the length of data left to hash is less than 512 bits */
  sha256_transf(ctx, shifted_message, block_nb);

  rem_len = new_len % SHA256_BLOCK_SIZE;

  memcpy(ctx->block, &shifted_message[block_nb << 6], rem_len);

  ctx->len = rem_len;
  ctx->tot_len += (block_nb + 1) << 6;
}

/** \brief final digest: calcolate digest on the last block and output the result */
void sha256_final(sha256_ctx *ctx, unsigned char *digest)
{
  t_uint32 block_nb;
  t_uint32 pm_len;
  t_uint32 len_b;

 #ifndef UNROLL_LOOPS
  t_int32 i;
 #endif

  block_nb = (1 + ((SHA256_BLOCK_SIZE - 9) < (ctx->len % SHA256_BLOCK_SIZE)));

  /* bit length of the input message */
  len_b = (ctx->tot_len + ctx->len) << 3;
  pm_len = block_nb << 6;

  /* pad last block: append a single "1" bit to the end of the message then
   * k "0" bits and last the end of the message, where k is defined such as the resulting
   * message is multiple of 512 bits */
  memset(ctx->block + ctx->len, 0, pm_len - ctx->len);
  ctx->block[ctx->len] = 0x80;
  UNPACK32(len_b, ctx->block + pm_len - 4);

  sha256_transf(ctx, ctx->block, block_nb);

  #ifndef UNROLL_LOOPS
  for (i = 0 ; i < 8; i++)
  {
    UNPACK32(ctx->h[i], &digest[i << 2]);
  }
  #else
    UNPACK32(ctx->h[0], &digest[ 0]);
    UNPACK32(ctx->h[1], &digest[ 4]);
    UNPACK32(ctx->h[2], &digest[ 8]);
    UNPACK32(ctx->h[3], &digest[12]);
    UNPACK32(ctx->h[4], &digest[16]);
    UNPACK32(ctx->h[5], &digest[20]);
    UNPACK32(ctx->h[6], &digest[24]);
    UNPACK32(ctx->h[7], &digest[28]);
  #endif /* !UNROLL_LOOPS */
}

/** \brief compute SHA-256 digest on a given message string */
void KeymgrSha256(const unsigned char *message, t_uint32 len,
    unsigned char *digest, sha256_ctx *ctx)
{
  sha256_init(ctx);
  sha256_update(ctx, message, len);
  sha256_final(ctx, digest);
}


/*---------------------------------------------------------------------------
 Copyright (c) 2002, Dr Brian Gladman, Worcester, UK.   All rights reserved.

 LICENSE TERMS

 The free distribution and use of this software in both source and binary
 form is allowed (with or without changes) provided that:

   1. distributions of this source code include the above copyright
      notice, this list of conditions and the following disclaimer;

   2. distributions in binary form include the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other associated materials;

   3. the copyright holder's name is not used to endorse products
      built using this software without specific written permission.

 ALTERNATIVELY, provided that this notice is retained in full, this product
 may be distributed under the terms of the GNU General Public License (GPL),
 in which case the provisions of the GPL apply INSTEAD OF those given above.

 DISCLAIMER

 This software is provided 'as is' with no explicit or implied warranties
 in respect of its properties, including, but not limited to, correctness
 and/or fitness for purpose.
 ---------------------------------------------------------------------------
 Issue Date: 26/08/2003

 This is an implementation of HMAC, the FIPS standard keyed hash function
*/


 /**************************************************
 * Function name : t_gig_error KeymgrHmac(const unsigned char *key, t_uint32 key_len,
 * const unsigned char *data, t_uint32 data_len, unsigned char *mac, t_uint32 mac_len)
 * Notes         : Public function
 *                 Pre-conditions:
 *                   1)Encryption Key Manager module has to be up and running
 *                   2)Secret key has to be generated
 *                 Post-conditions:
 *                   1)The input data is hashed with HMAC SHA-256 algorithm
 *
 **************************************************/
/** \brief HMAC-256 algorithm
 *
 * \param key point to the secret key
 * \param key_len byte size of the secret key
 * \param data point to data to hash
 * \param data_len length of data
 * \param mac point to memory where to store the hashed data
 * \param mac_len requested length of the hashed data
 *
 **************************************************/
void KeymgrHmac(const unsigned char *key, t_uint32 key_len,
    const unsigned char *data, t_uint32 data_len,
    unsigned char *mac, t_uint32 mac_len)
{
  unsigned char digest[SHA256_DIGEST_SIZE];
  /* HMAC context */
  hmac_ctx cx;
  t_uint32 i;

  /* initialise the HMAC context to zero */
  memset(&cx, 0, sizeof(hmac_ctx));

  /* input the HMAC key */
  /* check if the key has to be hashed  */
  if(key_len > HASH_INPUT_SIZE)
  {
    /* hash long key data */
    KeymgrSha256(key, key_len, cx.key, cx.ctx);
    /* set key length   */
    cx.klen = SHA256_DIGEST_SIZE;
  }
  else
  {
    /* otherwise store key data */
    memcpy(cx.key + cx.klen, key, key_len);
    /* update the key length count */
    cx.klen += key_len;
  }

  /* pad the key if necessary */
  memset(cx.key + cx.klen, 0, HASH_INPUT_SIZE - cx.klen);

  /* input the HMAC data */
  /* xor key value with ipad */
  for(i = 0; i < (HASH_INPUT_SIZE >> 2); ++i)
  {
    ((t_uint32 *)cx.key)[i] ^= 0x36363636;
  }

  /* start hash operation on the first block */
  sha256_init(cx.ctx);
  sha256_update(cx.ctx, cx.key, HASH_INPUT_SIZE);

  /* hash the input data */
  sha256_update(cx.ctx, data, data_len);

  /* complete the inner hash */
  sha256_final(cx.ctx, digest);

  /* set outer key value using opad and removing ipad */
  for(i = 0; i < (HASH_INPUT_SIZE >> 2); ++i)
  {
    ((t_uint32 *)cx.key)[i] ^= 0x36363636 ^ 0x5c5c5c5c;
  }

  /* perform the outer hash operation */
  sha256_init(cx.ctx);
  sha256_update(cx.ctx, cx.key, HASH_INPUT_SIZE);
  sha256_update(cx.ctx, digest, SHA256_DIGEST_SIZE);
  sha256_final(cx.ctx, digest);

  /* output the hash value */
  for(i = 0; i < mac_len; ++i)
  {
    mac[i] = digest[i];
  }

  return;
}


 /**************************************************
 * Function name : t_gig_error KeymgrPbkdf1Key(const unsigned char *pwd, const unsigned char *salt,
 * t_uint32 salt_len, t_uint32 iter, unsigned char *key, t_uint32 key_len);
 * Notes         : Public function
 *                 Pre-conditions:
 *                   1)Encryption Key Manager module has to be up and running
 *                 Post-conditions:
 *                   1)The AES Key is generated
 *
 **************************************************/
/** \brief Return a PKCS#5 PBKDF1 derived key
 * This implementation is derived from RFC 2898
 *
 * \param pwd point to the password to hash
 * \param pwd_len byte size of the password
 * \param salt point to the salt for PBKDF1
 * \param salt_len byte size of the salt
 * \param iter number of iteration to apply SHA-256
 * \param key space for the output key
 * \param key_len length of the output key
 *
 **************************************************/
void KeymgrPbkdf1Key(const unsigned char *pwd, t_uint32 pwd_len,
    const unsigned char *salt,  t_uint32 salt_len,
    t_uint32 iter, unsigned char *key, t_uint32 key_len)
{
  /* store the initial string to hash */
  unsigned char start[KEYMGR_MAX_PSW_SIZE+KEYMGR_SALT_SIZE];
  /* store intermediate hash results */
  unsigned char message[SHA256_DIGEST_SIZE];
  /* store the output of SHA-256 */
  unsigned char digest[SHA256_DIGEST_SIZE];
  /* SHA-256 context */
  sha256_ctx ctx;
  t_uint32 i;

  memset(start, 0, sizeof start);
  memcpy(start, pwd, pwd_len);
  if (salt_len != 0)
  {
    /* concatenate password with salt */
    memcpy(start + pwd_len, salt, salt_len);
  }

  KeymgrSha256(start, strlen((char *)start), message, &ctx);

  for(i = 1; i < iter; i++)
  {
    KeymgrSha256(message, SHA256_DIGEST_SIZE, digest, &ctx);
    memcpy(message, digest, SHA256_DIGEST_SIZE);
  }

  /* output the key value */
  for(i = 0; i < key_len; ++i)
  {
    key[i] = digest[i];
  }

  return;
}




/*@}*/
