/**
 * \addtogroup keymgr
 */
/*@{*/

/***************************************************
 * Header name: keymgr_crypto.h
 *
 ***************************************************/
/** \file keymgr_crypto.h
 *
 * \brief Exported functions for Cryptography algorightm support.
 *
 **************************************************/
/*
 * FIPS 180-2 SHA-224/256/384/512 implementation
 * Last update: 02/02/2007
 * Issue date:  04/30/2005
 *
 * Copyright (C) 2005, 2007 Olivier Gay <olivier.gay@a3.epfl.ch>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#ifndef KEYMGR_CRYPTO_H_
#define KEYMGR_CRYPTO_H_


/***************************************************
 *                 Public Defines Section
 ***************************************************/



/***************************************************
 *                 Public Constants Section
 ***************************************************/



/***************************************************
 *                 Public Typedefs Section
 ***************************************************/



/***************************************************
 *                 Local Constants Section
 ***************************************************/

/** \brief maximum length of the input password for NMK/DAK generation */
#define KEYMGR_MAX_PSW_SIZE   (64)
/** \brief minimum length of the input password NMK/DAK generation */
#define KEYMGR_MIN_PSW_SIZE   (12)

/** \brief iteration numbers for NMK generation using PBKDF1 */
#define KEYMGR_NMK_PBKDF1_N_ITER            (1000)
/** \brief iteration numbers for DAK generation using PBKDF1 */
#define KEYMGR_DAK_PBKDF1_N_ITER            (1000)
/** \brief iteration numbers for NID generation using PBKDF1 */
#define KEYMGR_NID_PBKDF1_N_ITER            (5)
/** \brief iteration numbers for AES key generation using PBKDF1 - used for GIGLE-defined MMs */
#define KEYMGR_GIGLE_KEY_PBKDF1_N_ITER      (99)
/** \brief iteration numbers for AES key generation using PBKDF1 - used for primitives */
#define KEYMGR_PRIMITIVE_KEY_PBKDF1_N_ITER  (100)

/** \brief size of the salt for PBKDF1 function */
#define KEYMGR_SALT_SIZE      (8)

/** \brief SHA-256 digest size */
#define SHA256_DIGEST_SIZE    (256/8)
/** \brief SHA-256 block size */
#define SHA256_BLOCK_SIZE     (512/8)

/** \brief maximum length of the HMAC secret key */
#define HASH_INPUT_SIZE       (SHA256_BLOCK_SIZE)


/***************************************************
 *                 Local Typedefs Section
 ***************************************************/

/** \brief SHA-256 context */
/* hold details of the current hash state */
typedef struct
{
  t_uint32 tot_len; /**< count the byte size of the entire message */
  unsigned int len; /**< count how many bytes we have processed in the current block */
  unsigned char block[2 * SHA256_BLOCK_SIZE]; /**< internal buffer */
  t_uint32 h[8];  /**< initial Hash value */
}
sha256_ctx;

/** \brief HMAC context */
typedef struct
{
  unsigned char key[HASH_INPUT_SIZE]; /**< secret key of the HMAC */
  sha256_ctx ctx[1];                  /**< SHA-256 context */
  unsigned long klen;                 /**< size of the sectre key */
}
hmac_ctx;


/***************************************************
 *                 Public Function Prototypes Section
 ***************************************************/

void KeymgrSha256(const unsigned char *message, t_uint32 len,
    unsigned char *digest, sha256_ctx *ctx);
void KeymgrHmac(const unsigned char *key /* secret key*/, t_uint32 key_len,
    const unsigned char *data /*counter*/, t_uint32 data_len, unsigned char *mac,
    t_uint32 mac_len);
void KeymgrPbkdf1Key(const unsigned char *pwd, t_uint32 pwd_len,
    const unsigned char *salt, t_uint32 salt_len,
    t_uint32 iter, unsigned char *key, t_uint32 key_len);

#else
#error "Header file __FILE__ has already been included!"
#endif

/*@}*/
