/////////////////////////////////////////////////////////////////////
////                                                             ////
////  WISHBONE SPI Flash memory map controller                   ////
////                                                             ////
////  Author: Hakan Fouren                                       ////
////          hakan.fouren@gigle.biz                             ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
////                                                             ////
//// Copyright (C) 2005 Gigle Semiconductor                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

//  SVN tag information
//
//  $Id$
//

`include "./defines/mbc_defines.v"

module mbc_spi_mm(
	// Syscon interface
	rst_i, clk_i,

	// Configuration bits
	spi_mmenable_i, spi_lsb_i, spi_txedge_i, spi_rxedge_i,

	// Master interface
	slv_cyc_i, slv_stb_i, slv_adr_i, slv_we_i,
	slv_dat_i, slv_sel_i,
	slv_dat_o, slv_err_o, slv_rty_o, slv_ack_o,

	// Slaves Interface
	mst_cyc_o, mst_stb_o, mst_adr_o, mst_we_o,
	mst_dat_o, mst_sel_o,
	mst_dat_i, mst_err_i, mst_rty_i, slv_ack_i,

	spi_running_i
	);

////////////////////////////////////////////////////////////////////
//
// Module IOs
//

// Syscon interface
input				rst_i;		// Reset
input				clk_i;		// Clock

// Configuration interface (from GPIO)
input				spi_mmenable_i;	// Memory map SPI Flash
input				spi_lsb_i;	// SPI transfers LSB first
input				spi_txedge_i;	// SPI edge for address writes
input				spi_rxedge_i;	// SPI edge for data reads

// Master Interface
input				slv_cyc_i;	// Cycle in progress (bus req.)
input				slv_stb_i;	// Strobe (command valid)

input	[`WB_ADDR_WIDTH-1:0]	slv_adr_i;	// Address
input				slv_we_i;	// Write enable
input	[`WB_DATA_WIDTH-1:0]	slv_dat_i;	// Write data
input	[`WB_SEL_WIDTH-1:0]	slv_sel_i;	// Byte strobes

output	[`WB_DATA_WIDTH-1:0]	slv_dat_o;	// Read data
output				slv_err_o;	// Transfer error
output				slv_rty_o;	// Transfer retry
output				slv_ack_o;	// Acknowledge (command end)

// Slaves Interface
output				mst_cyc_o;	// Cycle in progress (bus req.)
output				mst_stb_o;	// Strobe (command valid)

output	[`WB_ADDR_WIDTH-1:0]	mst_adr_o;	// Address
output				mst_we_o;	// Write enable
output	[`WB_DATA_WIDTH-1:0]	mst_dat_o;	// Write data
output	[`WB_SEL_WIDTH-1:0]	mst_sel_o;	// Byte strobes

input	[`WB_DATA_WIDTH-1:0]	mst_dat_i;	// Read data
input				mst_err_i;	// Transfer error
input				mst_rty_i;	// Transfer retry
input				slv_ack_i;	// Acknowledge (command end)

input				spi_running_i;	// SPI is active

////////////////////////////////////////////////////////////////////
//
// SPI access machine
//

reg				slv_cyc_mm;
reg	[`WB_ADDR_WIDTH-1:0]	slv_adr_mm;
reg				slv_we_mm;
reg	[`WB_DATA_WIDTH-1:0]	slv_dat_mm;

reg	[3:0]			mm_state;
`define	SLV_SPI_IDLE		4'b0000
`define	SLV_SPI_WR_SEL		4'b0111
`define	SLV_SPI_WR_ADR		4'b0110
`define	SLV_SPI_WR_ADR_CMD	4'b0101
`define	SLV_SPI_WR_ADR_GO	4'b0100
`define	SLV_SPI_WAIT_ADR	4'b1111
`define	SLV_SPI_WR_DAT_CMD	4'b1110
`define	SLV_SPI_WR_DAT_GO	4'b1101
`define	SLV_SPI_WAIT_DAT	4'b1100
`define	SLV_SPI_RD_D3		4'b1011
`define	SLV_SPI_RD_D2		4'b1010
`define	SLV_SPI_RD_D1		4'b1001
`define	SLV_SPI_RD_D0		4'b1000
`define	SLV_SPI_WR_DESEL	4'b0011
reg				mm_active;
reg				mm_stb;

assign	mst_cyc_o = (spi_mmenable_i || mm_active) 		? slv_cyc_mm : slv_cyc_i;
assign	mst_stb_o = ((spi_mmenable_i || mm_active) && mm_stb) 	? slv_cyc_mm : slv_stb_i;
assign  mst_adr_o = (spi_mmenable_i || mm_active) 		? slv_adr_mm : slv_adr_i;
assign  mst_we_o  = (spi_mmenable_i || mm_active) 		? slv_we_mm  : slv_we_i;
assign	mst_sel_o = (spi_mmenable_i || mm_active) 		? ~0         : slv_sel_i;
assign  mst_dat_o = (spi_mmenable_i || mm_active) 		? slv_dat_mm : slv_dat_i;

always @(posedge clk_i or posedge rst_i) begin
  if (rst_i) begin
    mm_state   <= #1 `SLV_SPI_IDLE;
    slv_cyc_mm <= #1 0;
    mm_active  <= #1 0;
    mm_stb     <= #1 1;
  end else begin
    case(mm_state)
      // synopsys parallel_case full_case
      `SLV_SPI_IDLE: begin
        slv_cyc_mm <= #1 0;
        mm_active  <= #1 0;
        mm_stb     <= #1 1;
        if (spi_mmenable_i && slv_cyc_i && slv_stb_i) begin
          mm_state   <= #1 `SLV_SPI_WR_SEL;
          slv_cyc_mm <= #1 1;
          mm_active  <= #1 1;
        end
      end
      `SLV_SPI_WR_SEL: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WR_ADR;
        end
      end
      `SLV_SPI_WR_ADR: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WR_ADR_CMD;
        end
      end
      `SLV_SPI_WR_ADR_CMD: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WR_ADR_GO;
        end
      end
      `SLV_SPI_WR_ADR_GO: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WAIT_ADR;
          slv_cyc_mm <= #1 0;
        end
      end
      `SLV_SPI_WAIT_ADR: begin
        slv_cyc_mm <= #1 0;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (!spi_running_i) begin
          mm_state   <= #1 `SLV_SPI_WR_DAT_CMD;
          slv_cyc_mm <= #1 1;
        end
      end
      `SLV_SPI_WR_DAT_CMD: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WR_DAT_GO;
        end
      end
      `SLV_SPI_WR_DAT_GO: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WAIT_DAT;
          slv_cyc_mm <= #1 0;
        end
      end
      `SLV_SPI_WAIT_DAT: begin
        slv_cyc_mm <= #1 0;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (!spi_running_i) begin
          mm_state   <= #1 `SLV_SPI_RD_D3;
          slv_cyc_mm <= #1 1;
          mm_stb     <= #1 0;
        end
      end
      `SLV_SPI_RD_D3: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 0;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_RD_D2;
        end
      end
      `SLV_SPI_RD_D2: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 0;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_RD_D1;
        end else if (!slv_cyc_i) begin
          mm_state   <= #1 `SLV_SPI_WR_DESEL;
          mm_stb     <= #1 1;
        end
      end
      `SLV_SPI_RD_D1: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 0;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_RD_D0;
        end else if (!slv_cyc_i) begin
          mm_state   <= #1 `SLV_SPI_WR_DESEL;
          mm_stb     <= #1 1;
        end
      end
      `SLV_SPI_RD_D0: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 0;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_WR_DESEL;
          mm_stb     <= #1 1;
        end else if (!slv_cyc_i) begin
          mm_state   <= #1 `SLV_SPI_WR_DESEL;
          mm_stb     <= #1 1;
        end
      end
      `SLV_SPI_WR_DESEL: begin
        slv_cyc_mm <= #1 1;
        mm_active  <= #1 1;
        mm_stb     <= #1 1;
        if (slv_ack_i) begin
          mm_state   <= #1 `SLV_SPI_IDLE;
          slv_cyc_mm <= #1 0;
          mm_active  <= #1 0;
        end
      end
    endcase
  end
end

always @(mm_state or slv_adr_i) begin
  case(mm_state)
    // synopsys parallel_case full_case
    `SLV_SPI_WR_SEL: begin
      slv_adr_mm <= `SPI_SELECT_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WR_ADR: begin
      slv_adr_mm <= `SPI_DATA_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WR_ADR_CMD: begin
      slv_adr_mm <= `SPI_CONTROL_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WR_ADR_GO: begin
      slv_adr_mm <= `SPI_CONTROL_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WAIT_ADR: begin
      slv_adr_mm <= `SPI_CONTROL_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WR_DAT_CMD: begin
      slv_adr_mm <= `SPI_CONTROL_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WR_DAT_GO: begin
      slv_adr_mm <= `SPI_CONTROL_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_WAIT_DAT: begin
      slv_adr_mm <= `SPI_CONTROL_ADDR;
      slv_we_mm  <= 1;
    end
    `SLV_SPI_RD_D3: begin
      slv_adr_mm <= `SPI_DATA_ADDR | slv_adr_i[`RAM_ADDR_OFFSET-1:0];
      slv_we_mm  <= 0;
    end
    `SLV_SPI_RD_D2: begin
      slv_adr_mm <= `SPI_DATA_ADDR | slv_adr_i[`RAM_ADDR_OFFSET-1:0];
      slv_we_mm  <= 0;
    end
    `SLV_SPI_RD_D1: begin
      slv_adr_mm <= `SPI_DATA_ADDR | slv_adr_i[`RAM_ADDR_OFFSET-1:0];
      slv_we_mm  <= 0;
    end
    `SLV_SPI_RD_D0: begin
      slv_adr_mm <= `SPI_DATA_ADDR | slv_adr_i[`RAM_ADDR_OFFSET-1:0];
      slv_we_mm  <= 0;
    end
    `SLV_SPI_WR_DESEL: begin
      slv_adr_mm <= `SPI_SELECT_ADDR;
      slv_we_mm  <= 1;
    end
    // synopsys translate_off
    default: begin
      slv_adr_mm <= 32'hXXXXXXXX;
      slv_we_mm  <= 1'bX;
    end
    // synopsys translate_on
  endcase
end

always @(mm_state) begin
  case(mm_state)
    // synopsys parallel_case full_case
    `SLV_SPI_WR_SEL:
      slv_dat_mm <= `FLASH_SELECT_VALUE;
    `SLV_SPI_WR_ADR:
      slv_dat_mm <= (slv_adr_i & `FLASH_ADDRESS_MASK) | `FLASH_READ_CMD;
    `SLV_SPI_WR_ADR_CMD:
      slv_dat_mm <= `FLASH_ADDRESS_CMD;
    `SLV_SPI_WR_ADR_GO:
      slv_dat_mm <= `FLASH_ADDRESS_CMD | `FLASH_GO_MASK;
    `SLV_SPI_WR_DAT_CMD:
      slv_dat_mm <= `FLASH_DATA_CMD;
    `SLV_SPI_WR_DAT_GO:
      slv_dat_mm <= `FLASH_DATA_CMD | `FLASH_GO_MASK;
    `SLV_SPI_WR_DESEL:
      slv_dat_mm <= `FLASH_DESELECT_VALUE;
    // synopsys translate_off
    default:
      slv_dat_mm <= 32'hXXXXXXXX;
    // synopsys translate_on
  endcase
end

////////////////////////////////////////////////////////////////////
//
// Wishbone machine
//

assign slv_dat_o = mst_dat_i;
assign slv_err_o = mst_err_i;
assign slv_rty_o = mst_rty_i;
assign slv_ack_o = (mm_active && slv_we_mm) ? 0 : slv_ack_i;

endmodule
